%      readme.m
%      nonlinear nonstationary time series autoregressive model 
%      estimation given the structural parameters

%      The matlab routine tvymr4estim performs estimation of a wide range  
%      of nonlinear nonstationary time series autoregressive model that   
%      depends on structural parameters as well as unknown coefficients

%      In general the model that tvymr4estim is able to estimate is

%      y(t) = w(y(t-1), y(t-2), ..., y(t-p)) + u(t),

%      where w is a function of past time series values up to lag p and  
%      u(t) is a white noise sequence with zero mean and variance s^2
%      independent of time. The function w assumes the form

%      w(y(t-1), ..., y(t-p)) = w_0 + w_1 y(t-1) + ... w_p y(t-p) 

%      and each of the p+1 functions w_0, w_1, ..., w_p may be written

%      w_j(t,y(t-d)) = a_j + b_j t + c_j G(t) + d_j y(t-d) + f_j t y(t-d) +
%      g_j y(t-d) G(t) +h_j H(y(t-d)) +k_j t H(y(t-d)) +m_j G(t) H(y(t-d)),

%      for j =0, ..., p. The function w_j depends on both time t and the 
%      state variable y(t-d), that is the time series observed at time t-d.
%      The parameter d is called the delay parameter. This kind of
%      structure allows non stationarity (as a function of t) and non
%      linearity (as a function of the state variable) be accounted for by
%      the overall model.
%      A mask vector v allows the user to specify the model that is
%      considered the best choice to fit the data. This vector has 9
%      binary entries, one for each of the coefficients a, b, c, d, f, g,
%      h, k, m, and an entry equal to one informes that the corresponding
%      coefficient is present in the model while an entry equal to zero
%      informs that the corresponding coefficient is missing.

%      We may use the routine tvymr4estim for estimating stationary linear
%      autoregressive models too. For instance, let us simulate the
%      following stationary linear first order autoregressive model 
%      y(t) = 2 + 0.9 y(t-1) + u(t), u(t) i. i. standard normal
n=1000; p=1;  
u=randn(n,1); y=zeros(n,1); 
y(1)=2.0+u(1); 
for txi=p+1:n, y(txi)=2.0 +0.9*y(txi-1)+u(txi);end,
%      We may plot this artificial time series. After a short warm-up it
%      displays a stationary behavior around its mean value
plot(1:1:n, y, '-m'),
%      The only non zero coefficient in the specification of the model is a
%      so that the mask vector has only a 1 as first entry and the
%      remaining entries are all zeros
v = [1 0 0 0 0 0 0 0 0];
%      The input arguments of tvymr4estim are (y, d, p, mmask, thres, ttau,
%      type2) but we need only y and p in this case and set mmask = v. 
%      Moreover, to avoid warnings and errors, set some unnecessary 
%      arguments to dummy values
d=1; thres=[min(y)-1, max(y)]; ttau=[0, n]; type2=[1,1];
%      Then launch
[phi, resid, rssq] =  tvymr4estim(y, d, p, v, thres, ttau, type2);
%      and obtain (resid is the column vector of residuals)
% >> phi
% phi =
%     1.8794
%     0.9082
% >> rssq
% rssq =
%   1.0615e+003
%     The coefficients are a_0 and a_1, true values are equal to 2 and 0.9
%     respectively. The residual sum of squares has been computed as 
%     resid'*resid, and, if divided by n, we obtain the figure 1.0615 which
%     is very close to the white noise variance.
%     For comparison we may use the routine estvar.m specifically written
%     for vector stationary linear autoregressive model estimation
yi=y'; [rssq,resid,phi]=estvar(yi, p);
%     and get an identical estimate of the autoregressive coefficient at 
%     lag 1
% >> phi
% phi =
%     0.9082
% >> rssq
% rssq =
%   1.0618e+003
%     Note that the routine estvar computes the mean instead of the
%     constant, so we have to transform the mean value 
mean_y= mean(y); a_0=(1-phi)*mean_y;
%     and get the constant a_0 close to 2.

%     In the preceding example, we have one structural parameter that we
%     assume known, i.e. the autoregressive order p, and two coefficients,
%     a_0 and a_1, that, conditional on p, are estimated by least squares.
%     As a first example of a nonlinear model let us consider the following
%     threshold autoregressive where the regimes are made dependent on the
%     state variable y(t-d)
%     y(t) = -0.5 + 0.7 y(t-1) - 0.5 y(t-2) + u(t), if y(t-d)<=0,
%             0.5 - 0.7 y(t-1) + 0.5 y(t-2) + u(t), otherwise,
%     where u(t) are a sequence of i. i. standard normal random variables
%     We may create an artificial sample
n = 1000; p = 2; d=1; nr = 2; threshold = 0.0;
phi= [ -0.5 0.7 -0.5;  0.5 -0.7  0.5];
u=randn(n,1); y=zeros(n,1); y(1:2) = u(1:2);
for txi=p+1:n,
    if y(txi-d) <= threshold, 
        y(txi) = -0.5 + 0.7*y(txi -1) -0.5*y(txi-2) + u(txi);
    else,
        y(txi) = 0.5 - 0.7*y(txi -1) +0.5*y(txi-2) + u(txi);
    end,
end,
%     Let's plot the simulated time series
plot(1:1:n, y, '-b'),
%     In order to estimate the coefficients by using the routine
%     tvymr4estim.m we have to set the mask vector properly. The only non
%     zero coefficient is h, so we define
v = [0 0 0 0 0 0 1 0 0];
%     Note that the coefficients h_0, h_1, and h_2, each one is a vector
%     with two entries, the first one for the coefficient to be assumed in 
%     the first regime and the second one for the coefficient in the second
%     regime. We may note that some coefficients are always scalar (a, b, 
%     d, f) while the remaining ones (c, g, h, k, m) in general are 
%     vectors, though in some instances they may be scalar too. 
%     For the present model, the function H that multiplies the coefficient
%     h is a step function, that needs thresholds to be available for its 
%     computation. The flag type2 has to inform the estimation routine that
%     the function of the state variable is a step function and this is
%     done by setting type2=[1,.] where the dot stands for a dummy
%     argument. It serves to speficy the form of the time break function
%     that is not present here. A value has to be specified, however, so we
%     may set 
type2=[1,1]; 
%     while only the first entry will be used actually.
%     A vector thres has to be arranged by completing the information on
%     thresholds with the lower and upper bounds of the state variable:
thres=[min(y)-1, threshold, max(y)];
%     The routine will compute the number of regimes automatically. The
%     last information to the estimating routine is the dummy array
ttau=[0, n]; 
%     Then we are ready to display the calling statement
[phi, resid, rssq] =  tvymr4estim(y, d, p, v, thres, ttau, type2);
%     The following estimates are obtained
% >> phi
% phi =
%    -0.6037
%     0.4351
%     0.6663
%    -0.6447
%    -0.5060
%     0.4062
% >> rssq,
% rssq =
%   883.2414
% >> 
%     The coefficients are listed according to the autoregressive lag, i.e.
%     the first two figures are the constant in the first and second regime
%     respectively, h_0(1)=-0.6037 and h_0(2)=0.4351 (true values were -0.5
%     and 0.5 respectively). Likewise, h_1(1)=0.6663 and h_1(2)=-0.6447 
%     (true: 0.7 and -0.7), and  h_2(1)=-0.5060 and h_2(2)=0.4062 (true: 
%     -0.5 and 0.5). The residual variance is 0.8832, while the sartificial
%     white noise variance was 1. The threshold autoregressive model is
%     more complicated than a linear autoregressive one, so we expect
%     rather accurate though somewhat less brilliant results.
%     Let's use for comparison the routine setar40estim.m for estimating a  
%     threshold autoregressive model
[yphi, resid, rssq] =  setar40estim(y, thres, d, p);
%     As we estimate the same coefficients using the same least squares 
%     method, the resulting esimate have to be the same, excepted possibly
%     a rightmost less significant digit. Results are similar but different
%     if we run the routines on artificial time series generated by the 
%     same model but using a different random number sequence.  In this  
%     case the artificial time series are identical and we obtain the same
%     results though in a different arrangement 
% >> yphi
% 
% yphi =
% 
%    -0.4081    0.7280   -0.4863
%     0.5873   -0.7484    0.4819
% 
% >> rssq
% 
% rssq =
% 
%   1.0582e+003
% 
% >> 

%     A model with a time break at t=n/2 and linear in the state variable
%     may be simulated as follows. 
n= 1000; p= 2; nt= 2; ttau=[0 n/2 n]; % threshold (time break) at t=n/2  
%     coeff. phi first row: before t=n/2, second row: after t=n/2
phi= [-0.2  0.5  0.2;  0.2  -0.5  -0.2];
u=randn(n,1); y=zeros(n,1); y(1:2) = u(1:2);
for txi=p+1:n,
    y(txi) = (-0.2 + 0.5*y(txi -1) +0.2*y(txi-2))*(txi<=n/2) + ...
        (0.2 - 0.5*y(txi -1) - 0.2*y(txi-2))*(txi>n/2) + u(txi);
end,
%     Let's plot the simulated time series
plot(1:1:n, y, '-b'),
%     In order to estimate the coefficients by using the routine
%     tvymr4estim.m we have to set the mask vector properly. The only non
%     zero coefficient is c, so we define
v = [0 0 1 0 0 0 0 0 0];
%     and specify that the function G that mulyiplies c is a step function
%     This may be done by setting type2(2)=1.
%     We have to set the dummy parameter d=1 and thres=[min(y)-1 max(y)]
%     and call the routine tvymr4estim.m with type2=[1,1], in this latter
%     vector only the second entry will be used actually.
d =1; thres=[min(y)-1 max(y)]; type2=[1,1];
[xphi, resid, xrssq] =  tvymr4estim(y, d, p, v, thres, ttau, type2);
%     For comparison let's use the routine setar41estim.m as well
[yphi, resid, yrssq] =  setar41estim(y, ttau, d, p);
%     We obtain identical results, only arranged differently
% >> xphi
% 
% xphi =
% 
%    -0.2717
%     0.2291
%     0.5243
%    -0.4569
%     0.1197
%    -0.2899
% 
% >> yphi
% 
% yphi =
% 
%    -0.2717    0.5243    0.1197
%     0.2291   -0.4569   -0.2899
% 
% >> xrssq
% 
% xrssq =
% 
%   1.0393e+003
% 
% >> yrssq
% 
% yrssq =
% 
%   1.0393e+003
% 
% >> 

%     Let us consider now a threshold model that accounts for both non
%     stationarity and non linearity. Non stationarity is modeled as a time
%     break occurring at time t and different linear autoregressive models 
%     have to be estimated before and after the time breaks. Likewise, the 
%     non linearity is modeled as a threshold autoregressive model with
%     coefficients for each regime. Though the estimation routine may
%     perform estimation in the presence of more than two regimes, both in
%     time and in levels, we assume an artificial time series with two
%     regimes in time and two regimes in levels. The time series data may
%     follow one of four linear autoregressive models if the state variable
%     is greater or less the given threshold y(t-d) and the time is before 
%     or after the time break. Structural parameters and coefficients 
%     combine those used for the threshold model and the time break model 
%     considered in the previous examples.
n = 1000;
% number of regimes is equal to the number of intervals bounds minus one
thres =[min(y)-1 0 max(y)]; nr=size(thres,2)-1; d=1;   
ttau = [0  n/2  n];  nt=size(ttau, 2)-1; 
% coefficients are given: (1) first state regime, before time break, (2)
% after time break, (3) second state regime, before time break, (4) after 
% time break  
phi= [ -0.5 0.7 -0.5;  -0.2  0.5  0.2;  0.5 -0.7  0.5; 0.2  -0.5  -0.2];
p = 2;
% the only coefficient (actually a vector of length nr*nt) is the one that
% multiplies the product of the two step function, for which type=1
v= [0 0 0 0 0 0 0 0 1]; type2=[1 1];
% simulation begins ...
y =  randn(n,1);  for it=max(p,d)+1:n, 
   yp = 0;
    for ir = 1:nr, for itt= 1:nt, 
          yp = yp+1;
        if  (y(it-1)>thres(ir) &  y(it-1)<=thres(ir+1)) & (it>ttau(itt) & it<=ttau(itt+1)),
            y(it) = y(it) + phi(yp,1)+ phi(yp,2)*y(it-1) + phi(yp,3)*y(it-2);  end,
        end, end,
    end, 
plot(1:1:n, y, '-b'), % artificial time series plot
% estimating the coefficients ...
[ephi, resid, rssq] =  tvymr4estim(y, d, p, v, thres, ttau, type2);
% we obtain the following
% >> ephi
% ephi =
%    -0.3842
%    -0.0113
%     0.4438
%     0.1387
%     0.7459
%     0.6271
%    -0.7180
%    -0.6335
%    -0.5443
%     0.2109
%     0.5933
%    -0.1803
% >> 
%     The results may be checked by using the routine setar4estim
[phi, resid, rssq] =  setar4estim(y, thres, ttau, d, p);
%     The estimates are the same, arranged in a different array
% >> phi
% phi =
%    -0.3842    0.7459   -0.5443
%    -0.0113    0.6271    0.2109
%     0.4438   -0.7180    0.5933
%     0.1387   -0.6335   -0.1803
% >> rssq
% rssq =
%   1.0460e+003

%     Other function of the data which are available in tvymr4estim.m are
%     the piecewise linear (type=2) and the logistic function (type=3). 
%     For instance, we may want to deal with an artificial time series with
%     varying parameters depending on a state variable and a time break 
%     both modeled according to a logistic function. 
%     In this case the structural parameters are slightly different as far
%     as the specification of the logistic is concerned. Let
%     H(y(t-d)) = 1/(1 + exp(-gamma(y(t-d)-r)))
%     The program has to be given gamma and r as a row vector thres=[gamma,
%     r] and the type has to be set type2(1)=3. Moreover, let
%     G(t) = 1/(1 + exp(-delta(t-t_0)))
%     The parameters delta and t_0 may be stored as the row vector
%     ttau=[delta, t_0] and type2(2)=3.
%     Let's produce an artificial time series from a STAR model of order 1
n = 1000; p= 1; d =1;
thres =[0.0  5]; %  thres(1)=  state STAR threshold, thres(2) =state gamma parameter
ttau = [n/2  0.1];  %  ttau(1)=  break STAR threshold, ttau(2) =break gamma parameter
phi= [ 0.0  0.0  0.0  0.5    -0.6   0.4]'; % phi(1,2,3)= constant, phi(4,5,6)=  lag 1 
y =  randn(n,1);  for it=max(p,d)+1:n, 
y(it) = y(it) + (phi(4) + phi(5)*gstar(it, ttau) +phi(6)*gstar(y(it -1), thres))*y(it-1);
end,
plot(1:1:n, y, '-b'), % artificial time series plot
% a estimates the constant, c the coefficient of G(t) and g the coefficient of H(y(t-d))
v = [1 0 1 0 0 0 1 0 0]; type2 =  [3  3]; 
[estphi, resid, rssq] =  tvymr4estim(y, d, p, v, thres, ttau, type2);
%     We obtain the following estimates
% >> estphi
% estphi =
%     0.2680
%    -0.1165
%    -0.3323
%     0.5915
%    -0.5725
%     0.3540
%     We may check the estimates provided by tvymr4estim using the routine
%     star4estim that is useful for STAR model estimation only
[phi, u, rssq] =  star4estim(y, d, p, thres, ttau); 
% >> phi
% phi =
%     0.2680
%    -0.3323
%    -0.1165
%     0.5915
%     0.3540
%    -0.5725
% >> rssq
% rssq =
%   989.0792
%     Notice that, unlike tvymr4estim, star4estim displays the estimate of 
%     the coefficient of the logistic in the state variable before that of 
%     the logistic as a function of time. 

%     The nonlinear functions may be combined as well to model different
%     behaviors in time and in the levels. For instance, consider the model
%     y(t) = {lambda + mu t + nu S(t)} sign(y(t-1)) y(t-1) + u(t)
%     where we assume a time break in t_0 and
%     S(t) = 0 if t<=t_0 and t-t_0 otherwise.
%     The model is a threshold autoregressive in the state variable y(t-1)
%     because sign(y(t-1)) =1 if y(t-1)>0 and -1 otherwise (we exclude
%     y(t-1)=0) and we have a threshold and two regimes depending on y(t-1)
%     The autoregressive coefficients vary according to time as well and 
%     lambda + mu t + nu S(t) is the coefficient of a piecewise
%     linear threshold autoregressive (PLTAR) model. 
%     Set lambda=1/2, mu= -3/n, and nu= 6/n. The, after some algebra, 
%     the coefficient of such PLTAR model may be written 
%     1/2 - (3/n) t if 0<t<=n/2, -5/2 + (3/n) t if n/2<t<=n.
%     This latter formula shows that the function that defines the 
%     coefficient is a continuous function made of two straight lines with
%     different slope that are connected at the point (n/2, -1).
%     Let's simulate this artificial time series
n = 1000; p =1; d = 1;
m=max(p, d); u= randn(n, 1); y = zeros(n ,1); y(1:m)= u(1:m);
for it = m+1:n, 
    y(it)= ((1/2 -(3/n)*it + (6/n)* max(0, it-n/2))*sign(y(it-d)))* ...
        y(it -1) + u(it);
end,
%     and plot its graphic
plot(1:1:n, y, '-b'), 
%     All term in model multiply H(y(t-d)), so the coefficients to be
%     estimated are h and m. This means that we have to set
v = [0 0 0 0 0 0 1 0 1]; type2 = [1 2];
%     We need a coefficient at location 7 (lambda) because the pltar model
%     coefficient accounts only for nt terms and a constant is not included
%     Then, type2(1)=1 informs that the nonlinearity in the state variable
%     is modeled as a step function and the time break is modeled as a
%     piecewise linear threshold model. We arrange the thresholds as
%     follows
thres =[min(y)-1 0 max(y)]; ttau = [0  n/2  n]; 
%     and may call the estimation routine
[estphi, resid, rssq] = tvymr4estim(y, d, p, v, thres, ttau, type2);
%     The estimated coefficients are
% >> estphi
% 
% estphi =
% 
%    -0.0966
%    -0.0739
%    -0.0001
%     0.0003
%    -0.0003
%     0.0007
%    -0.4879
%     0.6324
%     0.0028
%    -0.0060
%    -0.0026
%     0.0048
% 
% >> rssq
% 
% rssq =
% 
%   1.0511e+003
% 
% >> 
%     The first 6 estimated coefficients should be actually zero because we
%     didn't include a constant term in our model. However, tvymr4estim.m 
%     always include a constant term, and coefficients are estimated though
%     they are close to zero compared with the corresponding coefficient of
%     the lag 1 term. Let's compare the remaining coeffcients to the true 
%     model parameters. These latter are in sequence -0.5, 0.5, for the two
%     values of h whether H(y(t-d)) equals -1 or 1, then the m coefficients 
%     follow, 0.003, -0.006 if H(y(t-d))= -1, and -0.003, 0.006 if equals 1
%     The estimated sequence is -0.4879, 0.6324, and 0.0028, -0.006,
%     -0.0026, 0.0048, close enough to the true values. The residual
%     variance is equal to 1.0511, close to the error variance 1.

%     Knowledge of the structural parameters is needed to allow the 
%     estimation routine tvymr4estim to fit a nonstationary nonlinear model
%     conveniently. Otherwise we may find ourselves in one of the following 
%     cases :
%     (1) We know the parameters at least approximately, or we may choose
%     among few feasible values. Then we may estimate some models and
%     compare them in order to choose the best one, according to the
%     residual variance or, if we want to prefer the model with as fewer
%     parameters as possible, though maintaining the residual variance low,
%     we may use the AIC criterion or one of its variants and modifications
%     (2) We know little about the parameters but these latter are not many
%     and we may bound the set of values that each one may assume. Then a
%     grid search may be performed that allows us to locate the best
%     parameter set at reasinable computational cost.
%     (3) Parameters are many and we don't know too much about them.
%     Heuristic methods that are able to search large discrete space may be
%     employed such as simulated annealing or evolutionary computing. We 
%     are planning to provide routines which may perform this task. For 
%     each proposal set of parameters we may estimate the coefficients and 
%     a measure of goodness-of-fit that we want to optimize. The problem 
%     resides in devising an efficient search technique that may be 
%     considered in some sense better than a purely random search.

%     Details and theoretical properties of nonlinear nonstationary models
%     for time series may be found, for instance, in

%     Amado, C. and Terasvirta, T., 2008, Modelling conditional and
%     unconditional heteroskedasticity with smoothly time varying
%     structure, SSE/EFI Working paper in Economics and Finance 691,
%     Stockholm School of Economics, Stockholm.

%     Chen, R. and Tsay, R. S., 1993, Functional autoregressive models,
%     Journal of the American Statistical Association, 88, 298--308.

%     Davis, R., Lee, T. and Rodriguez-Yam, G., 2006, Structural break
%     estimation for nonstationary time series models, Journal of the 
%     American Statistical Association, 101, 223--239.

%     Lundberg, S., Terasvirta, T. and Van Dijk, D., 2003, Time-varying
%     smooth transition autoregressive models, Journal of Business and
%     Economic Statistics, 21, 104--121.

%     McAleer, M. and Medeiros, M. C., 2008, A multiple regime smooth
%     transition heterogeneous autoregressive model for long memory and
%     asymmetries, Journal of Econometrics, 147, 104--119.

%     Priestley, M. B., 1988, Nonlinear and nonstationary time series
%     analysis, Academic Press, London.

%     Tong, H., 1990, Nonlinear time series: a dynamical system approach,
%     Oxford University Press, Oxford.







